﻿namespace Ecng.Trading.Xaml
{
	using System;
	using System.Collections.Generic;
	using System.Windows.Input;

	using Ecng.Common;
	using Ecng.Collections;
	using Ecng.Trading.BusinessEntities;

	/// <summary>
	/// Визуальный контрол для стакана.
	/// </summary>
	public partial class MarketDepthControl
	{
		private sealed class Command : ICommand
		{
			private readonly MarketDepthControl _parent;
			private readonly MarketDepthControlAction _action;

			public Command(MarketDepthControl parent, MarketDepthControlAction action)
			{
				if (parent == null)
					throw new ArgumentNullException("parent");

				if (action == null)
					throw new ArgumentNullException("action");

				_parent = parent;
				_action = action;
			}

			void ICommand.Execute(object parameter)
			{
				_parent.ActionInvoking.SafeInvoke(_action);
			}

			bool ICommand.CanExecute(object parameter)
			{
				switch (_action.Type)
				{
					case MarketDepthControlActionTypes.BuyLimit:
						break;
					case MarketDepthControlActionTypes.SellLimit:
						break;
					case MarketDepthControlActionTypes.CancelGroup:
						break;
					case MarketDepthControlActionTypes.BuyMarket:
						break;
					case MarketDepthControlActionTypes.SellMarket:
						break;
					default:
						throw new ArgumentOutOfRangeException();
				}

				return true;
			}

			event EventHandler ICommand.CanExecuteChanged
			{
				add { throw new NotImplementedException(); }
				remove { throw new NotImplementedException(); }
			}
		}

		private sealed class MarketDepthControlActionList : Collection<MarketDepthControlAction>
		{
			private readonly MarketDepthControl _parent;

			public MarketDepthControlActionList(MarketDepthControl parent)
			{
				if (parent == null)
					throw new ArgumentNullException("parent");

				_parent = parent;
			}

			protected override void OnAdding(MarketDepthControlAction item)
			{
				if (item.Binding != null)
					throw new ArgumentException("Действие уже было ранее добавлено.", "item");

				var command = new Command(_parent, item);

				if (item.Key != Key.None)
					item.Binding = new KeyBinding(command, item.Key, item.ModifierKey);
				else
					item.Binding = new MouseBinding(command, new MouseGesture(item.MouseAction, item.ModifierKey));

				_parent.InputBindings.Add(item.Binding);
				base.OnAdding(item);
			}

			protected override void OnRemoving(MarketDepthControlAction item)
			{
				_parent.InputBindings.Remove(item.Binding);
				base.OnRemoving(item);
			}

			protected override void OnClearing()
			{
				_parent.InputBindings.Clear();
				base.OnClearing();
			}
		}

		/// <summary>
		/// Создать визуальный контрол для стакана.
		/// </summary>
		public MarketDepthControl()
		{
            InitializeComponent();
			this.Actions = new MarketDepthControlActionList(this);
		}

		/// <summary>
		/// Стакан.
		/// </summary>
		public MarketDepth Depth { get; set; }

		/// <summary>
		/// Событие о произошедшем действии, добавленное через <see cref="Actions"/>.
		/// </summary>
		public event Action<MarketDepthControlAction> ActionInvoking;

		/// <summary>
		/// Коллекция действий, ассоциированных с визуальным контролом для стакана.
		/// </summary>
		public ICollection<MarketDepthControlAction> Actions { get; private set; }
	}
}